@@ -7,20 +7,21 @@ import shlex
 import signal
 from collections import namedtuple
 from dataclasses import asdict
+from datetime import datetime
 from pathlib import Path
 
 import click
 from click.exceptions import MissingParameter, UsageError
 from pykdebugparser.pykdebugparser import PyKdebugParser
-from pymobiledevice3.services.device_arbitration import DtDeviceArbitration
 from termcolor import colored
 
 import pymobiledevice3
 from pymobiledevice3.cli.cli_common import print_json, Command, default_json_encoder, wait_return
-from pymobiledevice3.exceptions import DvtDirListError, ExtractingStackshotError
+from pymobiledevice3.exceptions import DvtDirListError, ExtractingStackshotError, UnrecognizedSelectorError
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.services.accessibilityaudit import AccessibilityAudit
 from pymobiledevice3.services.debugserver_applist import DebugServerAppList
+from pymobiledevice3.services.device_arbitration import DtDeviceArbitration
 from pymobiledevice3.services.dtfetchsymbols import DtFetchSymbols
 from pymobiledevice3.services.dvt.dvt_secure_socket_proxy import DvtSecureSocketProxyService
 from pymobiledevice3.services.dvt.instruments.activity_trace_tap import ActivityTraceTap, decode_message_format
@@ -194,13 +195,17 @@ def device_information(lockdown: LockdownClient, color):
     """ Print system information. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         device_info = DeviceInfo(dvt)
-        print_json({
-            'system': device_info.system_information(),
+        info = {
             'hardware': device_info.hardware_information(),
             'network': device_info.network_information(),
             'kernel-name': device_info.mach_kernel_name(),
             'kpep-database': device_info.kpep_database(),
-        }, colored=color)
+        }
+        try:
+            info['system'] = device_info.system_information()
+        except UnrecognizedSelectorError:
+            pass
+        print_json(info, colored=color)
 
 
 @dvt.command('netstat', cls=Command)
@@ -540,6 +545,7 @@ def dvt_oslog(lockdown: LockdownClient, color, pid):
                 image_name = os.path.basename(sender_image_path)
                 subsystem = message.subsystem
                 category = message.category
+                timestamp = datetime.now()
 
                 if pid is not None and message_pid != pid:
                     continue
@@ -550,13 +556,14 @@ def dvt_oslog(lockdown: LockdownClient, color, pid):
                     formatted_message = message.name
 
                 if color:
+                    timestamp = colored(str(timestamp), attrs=['bold'])
                     message_pid = colored(str(message_pid), 'magenta')
                     subsystem = colored(subsystem, 'green')
                     category = colored(category, 'green')
                     image_name = colored(image_name, 'yellow')
                     message_type = colored(message_type, 'cyan')
 
-                print(f'[{subsystem}][{category}][{message_pid}][{image_name}] '
+                print(f'[{timestamp}][{subsystem}][{category}][{message_pid}][{image_name}] '
                       f'<{message_type}>: {formatted_message}')
 
 
