@@ -14,17 +14,20 @@ def cli():
 @cli.command('list-devices')
 @click.option('--color/--no-color', default=True)
 @click.option('-u', '--usb', is_flag=True, help='show only usb devices')
-def list_devices(color, usb):
+@click.option('-n', '--network', is_flag=True, help='show only network devices')
+def list_devices(color, usb, network):
     """ list connected devices """
     connected_devices = []
     for device in usbmux.list_devices():
         udid = device.serial
 
-        if usb:
-            if ':' in udid:
-                continue
+        if usb and not device.is_usb:
+            continue
 
-        lockdown = LockdownClient(udid, autopair=False)
-        connected_devices.append(lockdown.all_values)
+        if network and not device.is_network:
+            continue
+
+        lockdown = LockdownClient(udid, autopair=False, connection_type=device.connection_type)
+        connected_devices.append(lockdown.device_info)
 
     print_json(connected_devices, colored=color)
